//
//  W2OController.h
//  W2O
//
//  Created by Woon Seung Yeo on 2/26/06.
//	Based on Apple's "RFCOMM_Open_SPP_Example" sample codes
//
//  Copyright 2006 CCRMA, Stanford University. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// Bluetooth headers:
#import <IOBluetooth/objc/IOBluetoothDevice.h>
#import <IOBluetooth/objc/IOBluetoothSDPUUID.h>
#import <IOBluetooth/objc/IOBluetoothRFCOMMChannel.h>
#import <IOBluetoothUI/objc/IOBluetoothDeviceSelectorController.h>

// OSC Client header:
#import "ObjCOSC/OSCPort.h"

@interface W2OController : NSObject {
    IBOutlet id _deviceNameField;
    IBOutlet id _macAddressField;

    IBOutlet id _closeButton;
    IBOutlet id _openButton;
    IBOutlet id _startButton;
    IBOutlet id _stopButton;
	
	IBOutlet id _IPField;
	IBOutlet id _portField;

	IBOutlet id _tiltAccelRawAddressField;
	IBOutlet id _tiltAccelDegAddressField;
	IBOutlet id _accelerationMagAddressField;
	
	IBOutlet id _tiltAccelRawButton;
	IBOutlet id _tiltAccelDegButton;
	IBOutlet id _accelerationMagButton;

	IBOutlet id _calibrationDataMaxForm;
	IBOutlet id _calibrationDataMinForm;
	
	// Bluetooth variables:
	IOBluetoothDevice *mBluetoothDevice;
	IOBluetoothRFCOMMChannel *mRFCOMMChannel;
	
	// OSC transmission
	OSCPort *_oscPort;	
	
	float _calibrationDataMinima[3];
	float _calibrationDataMaxima[3];
	float _calibrationDataMean[3];
	float _calibrationDataScalingAmp[3];
	
	float _g;
}

// Methods to interact with the menu:
- (IBAction)closeConnectonAction:(id)sender;
- (IBAction)openConnectionAction:(id)sender;
- (IBAction)startDataStream:(id)sender;
- (IBAction)stopDataStream:(id)sender;

// Calibration data values in the Preference Panel.
- (IBAction)setCalibrationMaxValue:(id)sender;
- (IBAction)setCalibrationMinValue:(id)sender;

// Delegate on window to know when the user closes the window
// so we can close everything.
- (BOOL)windowShouldClose:(id)sender;

// Methods to handle the Baseband and RFCOMM connection:
- (BOOL)openSerialPortProfile;
- (void)closeRFCOMMConnectionOnChannel:(IOBluetoothRFCOMMChannel*)channel;
- (void)closeDeviceConnectionOnDevice:(IOBluetoothDevice*)device;

// These are methods that are called when "things" happen on the
// bluetooth connection, read along and it will all be clearer:

// Called by the RFCOMM channel on us once the baseband and rfcomm connection is completed:
- (void)rfcommChannelOpenComplete:(IOBluetoothRFCOMMChannel*)rfcommChannel status:(IOReturn)error;

// Called by the RFCOMM channel on us when new data is received from the channel:
- (void)rfcommChannelData:(IOBluetoothRFCOMMChannel *)rfcommChannel data:(void *)dataPointer length:(size_t)dataLength;

// Called by the RFCOMM channel on us when something happens and the connection is lost:
- (void)rfcommChannelClosed:(IOBluetoothRFCOMMChannel *)rfcommChannel;

@end
